package com.qmth.sop.business.activiti.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.sop.business.bean.dto.FlowRejectSequenceDto;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.bean.result.FlowResult;
import com.qmth.sop.business.bean.result.FlowTablePropResult;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.entity.TFCustomFlow;
import com.qmth.sop.business.service.TFCustomFlowService;
import com.qmth.sop.common.contant.SpringContextHolder;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.TFCustomModelTypeEnum;
import com.qmth.sop.common.enums.TFCustomTypeEnum;
import com.qmth.sop.common.util.JacksonUtil;
import org.activiti.engine.delegate.event.ActivitiEntityEvent;
import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.delegate.event.ActivitiEventListener;
import org.activiti.engine.impl.persistence.entity.ResourceEntity;
import org.dom4j.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.Serializable;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @Description: 全局监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/2/8
 */
@Component
public class ProcessEventListener implements ActivitiEventListener, Serializable {
    private final static Logger log = LoggerFactory.getLogger(ProcessEventListener.class);

    @Override
    public void onEvent(ActivitiEvent event) {
        switch (event.getType()) {
            case PROCESS_STARTED:
                // 流程启动
                log.info("流程启动_PROCESS_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
            case PROCESS_COMPLETED:
                // 流程结束
                log.info("流程结束_PROCESS_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
            case TASK_COMPLETED:
                // 任务被完成了。它会在ENTITY_DELETE事件之前触发。当任务是流程一部分时，事件会在流程继续运行之前， 后续事件将是ACTIVITY_COMPLETE，对应着完成任务的节点。
                log.info("流程任务完成_TASK_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
//            case TASK_ASSIGNED:
//                log.info("流程分配人员_TASK_ASSIGNED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            case ENTITY_CREATED:
                ActivitiEntityEvent activitiEntityEvent = (ActivitiEntityEvent) event;
                if (activitiEntityEvent.getEntity() instanceof ResourceEntity) {
                    ResourceEntity resourceEntity = (ResourceEntity) activitiEntityEvent.getEntity();
                    log.info("ENTITY_CREATED,id:{},name:{},deploymentId:{},persistentState:{}", resourceEntity.getId(), resourceEntity.getName(), resourceEntity.getDeploymentId(), resourceEntity.getPersistentState());
                    String xml = new String(resourceEntity.getBytes(), StandardCharsets.UTF_8);
                    if (resourceEntity.getName().endsWith(SystemConstant.BPMN_PREFIX)) {
                        parseXml(xml, resourceEntity);
                    } else {
//                        log.info("ENTITY_CREATED,data:{}", data);
                        SystemConstant.FLOW_MAP.add(resourceEntity.getDeploymentId() + SystemConstant.FLOW_FORM_ID_SPACE + resourceEntity.getName(), xml);
                        if (SystemConstant.FLOW_MAP.containsKey(resourceEntity.getDeploymentId() + SystemConstant.BPMN_PREFIX)) {
                            parseXml(SystemConstant.FLOW_MAP.get(resourceEntity.getDeploymentId() + SystemConstant.BPMN_PREFIX).get(0), resourceEntity);
                        }
                    }
                }
                break;
            default:
//                log.info("default->type:{},ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getType(), event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
        }
    }

    @Override
    public boolean isFailOnException() {
        return false;
    }

    /**
     * 解析xml
     *
     * @param xml
     * @param resourceEntity
     */
    protected void parseXml(String xml, ResourceEntity resourceEntity) {
//        log.info("ENTITY_CREATED,xml:{}", xml);
        try {
            Document doc = DocumentHelper.parseText(xml);
            Element rootElement = doc.getRootElement();
            // 通过element对象的elementIterator方法获取迭代器
            Iterator iterator = rootElement.elementIterator();
            LinkedHashMap<String, FlowTaskResult> setupMap = new LinkedHashMap<>();//流程审批步骤map
            TFCustomTypeEnum customTypeEnum = null;
            Map<String, List<FlowRejectSequenceDto>> approveRejectMap = null;
            while (iterator.hasNext()) {
                Element element = (Element) iterator.next();
                List<Attribute> attributes = element.attributes();
                for (Attribute attr : attributes) {
//                    log.info("属性：{},---值：{}", attr.getName(), attr.getValue());
                    if (Objects.equals(attr.getName(), SystemConstant.ID) &&
                            (Objects.equals(attr.getValue(), TFCustomTypeEnum.OFFICE_SOP_FLOW.name())
                                    || Objects.equals(attr.getValue(), TFCustomTypeEnum.CLOUD_MARK_SOP_FLOW.name())
                                    || Objects.equals(attr.getValue(), TFCustomTypeEnum.DING_EXCEPTION_FLOW.name())
                                    || Objects.equals(attr.getValue(), TFCustomTypeEnum.PROJECT_EXCHANGE_FLOW.name())
                                    || Objects.equals(attr.getValue(), TFCustomTypeEnum.QUALITY_PROBLEM_FLOW.name()))) {
                        customTypeEnum = TFCustomTypeEnum.valueOf(attr.getValue());
                        break;
                    }
                }
                Iterator itt = element.elementIterator();
                while (itt.hasNext()) {
                    FlowTaskResult flowTaskResult = new FlowTaskResult();
                    Element node = (Element) itt.next();
                    attributes = node.attributes();
//                    log.info("节点：{},---值：{}", node.getName(), node.getStringValue());
                    if (Objects.equals(node.getName(), SystemConstant.USER_TASK) || Objects.equals(node.getName(), SystemConstant.START_EVENT) || Objects.equals(node.getName(), SystemConstant.END_EVENT)) {
                        for (Attribute attr : attributes) {
//                            log.info("节点属性：{},---节点值：{}", attr.getName(), attr.getValue());
                            if (Objects.equals(node.getName(), SystemConstant.USER_TASK) || Objects.equals(node.getName(), SystemConstant.END_EVENT)) {
                                if (Objects.equals(attr.getName(), SystemConstant.ID)) {
                                    flowTaskResult.setTaskKey(attr.getValue());
                                    flowTaskResult.setSetup(Integer.valueOf(attr.getValue().substring(attr.getValue().length() - 1, attr.getValue().length())));
                                    setupMap.put(attr.getValue(), flowTaskResult);
                                } else if (Objects.equals(attr.getName(), SystemConstant.NAME)) {
                                    flowTaskResult.setTaskName(attr.getValue());
                                } else if (Objects.equals(attr.getName(), SystemConstant.FORM_KEY) && SystemConstant.FLOW_MAP.containsKey(resourceEntity.getDeploymentId() + SystemConstant.FLOW_FORM_ID_SPACE + attr.getValue())) {
                                    if (Objects.equals(node.getName(), SystemConstant.USER_TASK) || Objects.equals(node.getName(), SystemConstant.END_EVENT)) {
                                        flowTaskResult.setFormKey(attr.getValue());
                                        List<String> list = SystemConstant.FLOW_MAP.get(resourceEntity.getDeploymentId() + SystemConstant.FLOW_FORM_ID_SPACE + attr.getValue());
                                        String data = list.get(list.size() - 1);
                                        List<FlowFormWidgetResult> listData = JSONArray.parseArray(data, FlowFormWidgetResult.class);
                                        for (FlowFormWidgetResult f : listData) {
                                            f.setFormId(flowTaskResult.getTaskKey() + SystemConstant.FLOW_FORM_ID_SPACE + f.getFormId());
                                            f.setFormName(flowTaskResult.getTaskKey() + SystemConstant.FLOW_FORM_ID_SPACE + f.getFormName());
                                            if (!CollectionUtils.isEmpty(f.getTablePropList())) {
                                                for (FlowTablePropResult t : f.getTablePropList()) {
                                                    t.setTdId(flowTaskResult.getTaskKey() + SystemConstant.FLOW_FORM_ID_SPACE + t.getWidgetId() + SystemConstant.FLOW_FORM_ID_SPACE + t.getTdId());
                                                    t.setTdName(flowTaskResult.getTaskKey() + SystemConstant.FLOW_FORM_ID_SPACE + t.getWidgetId() + SystemConstant.FLOW_FORM_ID_SPACE + t.getTdName());
                                                }
                                            }
                                        }
                                        flowTaskResult.setFormProperty(listData);
                                    }
                                }
                            } else {
                                List<String> list = SystemConstant.FLOW_MAP.get(resourceEntity.getDeploymentId() + SystemConstant.FLOW_FORM_ID_SPACE + attr.getValue());
                                if (!CollectionUtils.isEmpty(list)) {
                                    String data = list.get(list.size() - 1);
                                    if (Objects.nonNull(data)) {
                                        JSONObject jsonObject = JSONObject.parseObject(data);
                                        if (Objects.nonNull(jsonObject.getJSONObject(SystemConstant.ARRROVE_REJECT_MAP))) {
                                            JSONObject object = jsonObject.getJSONObject(SystemConstant.ARRROVE_REJECT_MAP);
                                            approveRejectMap = new LinkedHashMap<>(object.size());
                                            for (Map.Entry<String, Object> entry : object.entrySet()) {
                                                approveRejectMap.put(entry.getKey(), (List<FlowRejectSequenceDto>) entry.getValue());
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            List<Map.Entry<String, FlowTaskResult>> entryList = new ArrayList<Map.Entry<String, FlowTaskResult>>(setupMap.entrySet());
            //比较器
            Collections.sort(entryList, new Comparator<Map.Entry<String, FlowTaskResult>>() {
                @Override
                public int compare(Map.Entry<String, FlowTaskResult> o1, Map.Entry<String, FlowTaskResult> o2) {
                    if (o2.getValue().getSetup() < o1.getValue().getSetup()) {
                        return 1;
                    } else if (o2.getValue().getSetup() > o1.getValue().getSetup()) {
                        return -1;
                    } else {
                        return 0;
                    }
                }
            });
            setupMap.clear();
            Iterator<Map.Entry<String, FlowTaskResult>> iter = entryList.iterator();
            Map.Entry<String, FlowTaskResult> tmpEntry = null;
            while (iter.hasNext()) {
                tmpEntry = iter.next();
                FlowTaskResult flowTaskResult = tmpEntry.getValue();
                if (!CollectionUtils.isEmpty(approveRejectMap)) {
                    if (approveRejectMap.containsKey(tmpEntry.getKey())) {
                        flowTaskResult.setApproveRejectList(approveRejectMap.get(tmpEntry.getKey()));
                    }
                }
                setupMap.put(tmpEntry.getKey(), flowTaskResult);
            }
            FlowResult flowResult = new FlowResult(resourceEntity.getDeploymentId(), resourceEntity.getName(), setupMap);
//            log.info("flowResult:{}", JacksonUtil.parseJson(flowResult));
            TFCustomFlowService tfCustomFlowService = SpringContextHolder.getBean(TFCustomFlowService.class);
            TFCustomFlow tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getFlowDeploymentId, flowResult.getFlowDeploymentId()));
            if (Objects.isNull(tfCustomFlow)) {
                TFCustomFlow maxTfCustomFlow = tfCustomFlowService.findMaxVersion(null, flowResult.getFlowName(), customTypeEnum);
                tfCustomFlow = new TFCustomFlow(null, flowResult.getFlowName(), customTypeEnum, flowResult.getFlowDeploymentId(), TFCustomModelTypeEnum.USER_FIXED);
                flowResult.setId(tfCustomFlow.getId());
                if (Objects.nonNull(maxTfCustomFlow)) {
                    tfCustomFlow.setVersion(maxTfCustomFlow.getVersion() + 1);
                } else {
                    tfCustomFlow.setVersion(1);
                }
                flowResult.setVersion(tfCustomFlow.getVersion());
            } else {
                flowResult.setId(tfCustomFlow.getId());
                flowResult.setVersion(tfCustomFlow.getVersion());
                tfCustomFlow.setUpdateTime(System.currentTimeMillis());
            }
//            if (!CollectionUtils.isEmpty(approveRejectMap)) {
//                flowResult.setApproveRejectMap(approveRejectMap);
//            }
            tfCustomFlow.setFlowProcessVar(JacksonUtil.parseJson(flowResult));
            tfCustomFlowService.saveOrUpdate(tfCustomFlow);
        } catch (DocumentException e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
        SystemConstant.FLOW_MAP.add(resourceEntity.getDeploymentId() + SystemConstant.BPMN_PREFIX, xml);
    }
}
