package com.qmth.sop.business.activiti.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.result.*;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.SysUserMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.lock.MemoryLock;
import com.qmth.sop.common.util.GsonUtil;
import com.qmth.sop.common.util.JacksonUtil;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowNode;
import org.activiti.bpmn.model.UserTask;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.history.HistoricIdentityLink;
import org.activiti.engine.history.HistoricTaskInstance;
import org.activiti.engine.impl.bpmn.behavior.MultiInstanceActivityBehavior;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.IdentityLink;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipInputStream;

/**
 * @Description: activiti service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/12
 */
@Service
public class ActivitiServiceImpl implements ActivitiService {
    private final static Logger log = LoggerFactory.getLogger(ActivitiServiceImpl.class);

    @Resource
    RepositoryService repositoryService;

    @Resource
    TaskService taskService;

    @Resource
    TFCustomFlowService tfCustomFlowService;

    @Resource
    RuntimeService runtimeService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    SysUserMapper sysUserMapper;

    @Resource
    MemoryLock memoryLock;

    @Resource
    HistoryService historyService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    SysUserService sysUserService;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    SequenceService sequenceService;

    /**
     * 根据deploymentId查找processDefinitionId
     *
     * @param deploymentId
     * @return
     */
    @Override
    public String findProcessDefinitionIdByDeploymentId(String deploymentId) {
        return sysUserMapper.findProcessDefinitionIdByDeploymentId(deploymentId);
    }

    /**
     * 上传流程
     *
     * @param file
     * @return
     * @throws IOException
     */
    @Override
    public Map<String, Object> upload(MultipartFile file) throws IOException {
        basicAttachmentService.validateAttachment(file);
        Map<String, Object> map = new HashMap<>();
        DeploymentBuilder builder = repositoryService.createDeployment();
        ZipInputStream zip = new ZipInputStream(file.getInputStream());
        builder.addZipInputStream(zip);
        Map<String, Object> mapData = formPropertiesGet(builder.deploy().getId(), null, null);
        map.put(file.getName(), mapData);
        return map;
    }

    /**
     * 部署流程
     *
     * @param tfCustomTypeEnumList
     * @return
     * @throws IOException
     */
    @Override
    public Map<String, Object> createDeployment(List<TFCustomTypeEnum> tfCustomTypeEnumList) throws IOException {
        Map<String, Object> map = new HashMap<>();
        for (TFCustomTypeEnum tfCustomTypeEnum : tfCustomTypeEnumList) {
            List<TFCustomFlow> tfCustomFlowList = tfCustomFlowService.list(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getType, tfCustomTypeEnum));
            if (CollectionUtils.isEmpty(tfCustomFlowList)) {
                DeploymentBuilder builder = repositoryService.createDeployment();
                ClassPathResource resource = new ClassPathResource(tfCustomTypeEnum.getFileName());
                ZipInputStream zip = new ZipInputStream(resource.getInputStream());
                builder.addZipInputStream(zip);
                Map<String, Object> mapData = formPropertiesGet(builder.deploy().getId(), null, null);
                map.put(tfCustomTypeEnum.name(), mapData);
            } else {
                Map<String, Object> mapData = formPropertiesGet(tfCustomFlowList.get(0).getFlowDeploymentId(), null, null);
                map.put(tfCustomTypeEnum.name(), mapData);
            }
        }
        return map;
    }

    /**
     * 审批任务
     *
     * @param flowApproveParam
     * @return
     */
    @Override
    public Map<String, Object> taskApprove(FlowApproveParam flowApproveParam) throws InterruptedException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = new HashMap<>();
        List<String> approveUserIds = flowApproveParam.getApproveUserIds();
        if (CollectionUtils.isEmpty(approveUserIds) && (flowApproveParam.getApprove() == FlowApprovePassEnum.START || flowApproveParam.getApprove() == FlowApprovePassEnum.DRAFT || flowApproveParam.getApprove() == FlowApprovePassEnum.PASS)) {
            approveUserIds = new ArrayList<>();
            approveUserIds.add(String.valueOf(sysUser.getId()));//否则是当前登录用户
        }
        String processFlowId = null;
        TFCustomFlowEntity tfCustomFlowEntity = null;
        TFFlowApprove tfFlowApprove = null;
        Task taskTemp = null;
        FlowResult flowResult = null;
        if (Objects.nonNull(flowApproveParam.getFlowDeploymentId()) && Objects.isNull(flowApproveParam.getTaskId())) {
            TFCustomFlow tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getFlowDeploymentId, flowApproveParam.getFlowDeploymentId()));
            Optional.ofNullable(tfCustomFlow.getFlowProcessVar()).orElseThrow(() -> ExceptionResultEnum.FLOW_PROCESS_VAR_NO_DATA.exception());

            map.put(SystemConstant.DEFAULT_ASSIGNEE_LIST + 1, approveUserIds);
            ProcessInstance processInstance = runtimeService.startProcessInstanceById(tfCustomFlow.getFlowDefinitionId(), map);
            processFlowId = processInstance.getId();

            String code = sequenceService.createFlowCode(tfCustomFlow.getType());
            tfCustomFlowEntity = new TFCustomFlowEntity(tfCustomFlow.getId(), sysUser.getId(), code);
            tfFlowApprove = new TFFlowApprove(sysUser.getOrgId(), Long.parseLong(approveUserIds.get(approveUserIds.size() - 1)), FlowStatusEnum.valueOf(flowApproveParam.getApprove().name()), null);
            flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
        } else if (Objects.nonNull(flowApproveParam.getTaskId())) {
            taskTemp = taskService.createTaskQuery().taskId(String.valueOf(flowApproveParam.getTaskId())).singleResult();
            Optional.ofNullable(taskTemp).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_NO_DATA.exception());
            processFlowId = taskTemp.getProcessInstanceId();
        }

        boolean lock = memoryLock.lock(SystemConstant.LOCK_FLOW_TASK_PREFIX + processFlowId, processFlowId, SystemConstant.LOCK_FLOW_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在审批中，请稍候再试！");
        }
        try {
            List<Task> taskList = new ArrayList<>();
            if (Objects.isNull(taskTemp)) {
                taskList = taskService.createTaskQuery().processInstanceId(processFlowId).list();
            } else {
                tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, Long.parseLong(processFlowId)));
                Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());
                flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);

                tfFlowApprove = tfFlowApproveService.getOne(new QueryWrapper<TFFlowApprove>().lambda().eq(TFFlowApprove::getFlowId, Long.parseLong(processFlowId)));
                Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.FLOW_APPROVE_NO_DATA.exception());
                taskList.add(taskTemp);
            }
            List<TFFlowLog> tfFlowLogList = new ArrayList<>();
            LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();

            tfCustomFlowEntity.setFlowId(Long.parseLong(processFlowId));
            tfFlowApprove.setFlowId(Long.parseLong(processFlowId));
            tfFlowApprove.setApproveMaxSetup(setupMap.size());
            for (Task task : taskList) {
                BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
                FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(task.getTaskDefinitionKey());
                FlowTaskResult currFlowTaskResult = null, nextFlowTaskResult = null, formDataFlowTaskResult = null;
                Integer nrOfCompletedInstances = null, nrOfInstances = null;
                boolean multiInstance = false;
                if (currFlow instanceof UserTask) {
                    if (currFlow.getBehavior() instanceof MultiInstanceActivityBehavior) {
                        nrOfCompletedInstances = (Integer) taskService.getVariable(task.getId(), SystemConstant.NR_OF_COMPLETED_INSTANCES);
                        nrOfCompletedInstances++;
                        nrOfInstances = (Integer) taskService.getVariable(task.getId(), SystemConstant.NR_OF_INSTANCES);
                        multiInstance = true;
                    }
                    currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);
                    if (Objects.nonNull(flowApproveParam.getFormProperties())) {
                        formDataFlowTaskResult = JSONObject.parseObject(flowApproveParam.getFormProperties(), FlowTaskResult.class);
                    }
                    int currSetup = currFlowTaskResult.getSetup().intValue();
                    if (currSetup != 1 && flowApproveParam.getApprove() == FlowApprovePassEnum.START) {
                        throw ExceptionResultEnum.ERROR.exception("只有流程第一步才需要提交！");
                    }
                    if (currSetup != 2 && flowApproveParam.getApprove() == FlowApprovePassEnum.CANCEL) {
                        throw ExceptionResultEnum.ERROR.exception("只有流程第二步才能撤回！");
                    }
                    //流程流水日志
                    TFFlowLog tfFlowLog = new TFFlowLog(sysUser.getOrgId(), sysUser.getId(), sysUser.getId(),
                            currFlowTaskResult.getSetup(), Long.parseLong(processFlowId),
                            Long.parseLong(task.getId()), tfCustomFlowEntity.getId(), approveUserIds.toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                    if (flowApproveParam.getApprove() == FlowApprovePassEnum.PASS || flowApproveParam.getApprove() == FlowApprovePassEnum.DRAFT || flowApproveParam.getApprove() == FlowApprovePassEnum.START) {
                        if (Objects.nonNull(flowApproveParam.getSetup())) {
                            currSetup = flowApproveParam.getSetup();
                        } else {
                            currSetup = currSetup == setupMap.size() - 1 ? 0 : currSetup + 1;
                        }
                        nextFlowTaskResult = this.getNextFlowTaskResult(nrOfCompletedInstances,
                                nrOfInstances,
                                currFlowTaskResult,
                                nextFlowTaskResult,
                                currSetup,
                                setupMap,
                                flowApproveParam.getApprove());

                        this.approvePass(currFlowTaskResult,
                                setupMap,
                                multiInstance,
                                nrOfCompletedInstances,
                                nrOfInstances,
                                tfFlowApprove,
                                tfFlowLog,
                                nextFlowTaskResult,
                                map,
                                flowApproveParam.getApprove(),
                                flowApproveParam.getApproveRemark());
                        map.put(SystemConstant.DEFAULT_ASSIGNEE_LIST + nextFlowTaskResult.getSetup(), approveUserIds);
                    } else if (flowApproveParam.getApprove() == FlowApprovePassEnum.REJECT || flowApproveParam.getApprove() == FlowApprovePassEnum.CANCEL) {
                        if (Objects.nonNull(flowApproveParam.getSetup())) {
                            currSetup = flowApproveParam.getSetup();
                        } else {
                            currSetup = currSetup - 1;
                        }

                        nextFlowTaskResult = this.getNextFlowTaskResult(nrOfCompletedInstances,
                                nrOfInstances,
                                currFlowTaskResult,
                                nextFlowTaskResult,
                                currSetup,
                                setupMap,
                                flowApproveParam.getApprove());

                        this.approveReject(multiInstance,
                                map,
                                currFlowTaskResult,
                                nextFlowTaskResult,
                                tfFlowApprove,
                                tfFlowLog,
                                flowApproveParam.getApprove(),
                                flowApproveParam.getApproveRemark());
                    }
                    map.put(SystemConstant.APPROVE, nextFlowTaskResult.getTaskKey());
                    map.put(SystemConstant.DEFAULT_ASSIGNEE_LIST + nextFlowTaskResult.getSetup(), approveUserIds);
                    if (Objects.nonNull(formDataFlowTaskResult) && Objects.equals(task.getTaskDefinitionKey(), formDataFlowTaskResult.getTaskKey())) {
                        setupMap.put(task.getTaskDefinitionKey(), formDataFlowTaskResult);
                    }
                    flowResult.setSetupMap(setupMap);
                    tfCustomFlowEntity.setFlowProcessVar(JacksonUtil.parseJson(flowResult));
                    if (flowApproveParam.getApprove() == FlowApprovePassEnum.START || flowApproveParam.getApprove() == FlowApprovePassEnum.PASS
                            || flowApproveParam.getApprove() == FlowApprovePassEnum.CANCEL || flowApproveParam.getApprove() == FlowApprovePassEnum.REJECT) {
                        if (flowApproveParam.getApprove() == FlowApprovePassEnum.CANCEL || flowApproveParam.getApprove() == FlowApprovePassEnum.REJECT) {
                            List<HistoricTaskInstance> historicTaskInstanceList = historyService.createHistoricTaskInstanceQuery()
                                    .processInstanceId(task.getProcessInstanceId()).orderByHistoricTaskInstanceEndTime().desc().list();
                            if (!CollectionUtils.isEmpty(historicTaskInstanceList)) {
                                Set<String> approveIdSet = new LinkedHashSet<>();
                                for (int i = historicTaskInstanceList.size() - 1; i >= 0; i--) {
                                    HistoricTaskInstance historicTaskInstance = historicTaskInstanceList.get(i);
                                    if (Objects.equals(historicTaskInstance.getTaskDefinitionKey(), nextFlowTaskResult.getTaskKey())) {
                                        if (Objects.nonNull(historicTaskInstance.getAssignee())) {
                                            approveIdSet.add(historicTaskInstance.getAssignee());
                                        } else {
                                            List<HistoricIdentityLink> historicIdentityLinkList = historyService.getHistoricIdentityLinksForTask(historicTaskInstance.getId());
                                            if (!CollectionUtils.isEmpty(historicIdentityLinkList)) {
                                                for (HistoricIdentityLink h : historicIdentityLinkList) {
                                                    approveIdSet.add(h.getUserId());
                                                }
                                            }
                                        }
                                    }
                                }
                                map.put(SystemConstant.DEFAULT_ASSIGNEE_LIST + nextFlowTaskResult.getSetup(), new ArrayList<>(approveIdSet));
                            }
                        }
                        taskService.complete(task.getId(), map);
                        if (flowApproveParam.getApprove() == FlowApprovePassEnum.START || flowApproveParam.getApprove() == FlowApprovePassEnum.PASS) {
                            List<Task> tasks = taskService.createTaskQuery().processInstanceId(processFlowId).list();
                            if (!CollectionUtils.isEmpty(tasks)) {
                                Set<String> approveIdSet = new LinkedHashSet<>();
                                for (Task t : tasks) {
                                    if (Objects.isNull(t.getAssignee())) {
                                        List<IdentityLink> identityLinkList = taskService.getIdentityLinksForTask(t.getId());
                                        if (!CollectionUtils.isEmpty(identityLinkList)) {
                                            for (IdentityLink i : identityLinkList) {
                                                approveIdSet.add(i.getUserId());
                                            }
                                        }
                                    } else {
                                        approveIdSet.add(t.getAssignee());
                                    }
                                }
                                tfFlowLog.setPendApproveId(approveIdSet.toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                            }
                        }
                    }
                    tfFlowLogList.add(tfFlowLog);
                }
            }
            tfCustomFlowEntityService.saveOrUpdate(tfCustomFlowEntity);
            tfFlowApproveService.saveOrUpdate(tfFlowApprove);
            tfFlowLogService.saveBatch(tfFlowLogList);
            map.put(SystemConstant.FLOW_APPROVE, tfFlowApprove);
            map.put(SystemConstant.CUSTOM_FLOW_ENTITY, tfCustomFlowEntity);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            memoryLock.unlock(SystemConstant.LOCK_FLOW_TASK_PREFIX + processFlowId);
        }
        return map;
    }

    /**
     * 获取流程表单属性接口
     *
     * @param flowDeploymentId
     * @param flowId
     * @param taskId
     * @return
     */
    @Override
    public Map<String, Object> formPropertiesGet(String flowDeploymentId,
                                                 Long flowId,
                                                 Long taskId) {
        Map<String, Object> map = new HashMap<>();
        FlowResult flowResult = null;
        TFCustomFlow tfCustomFlow = null;
        List<Task> taskList = null;
        if (Objects.nonNull(flowDeploymentId)) {
            tfCustomFlow = tfCustomFlowService.getOne(new QueryWrapper<TFCustomFlow>().lambda().eq(TFCustomFlow::getFlowDeploymentId, flowDeploymentId));
            Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.FLOW_DEPLOYMENT_NO_DATA.exception());
            if (Objects.isNull(tfCustomFlow.getFlowDefinitionId())) {
                String processDefinitionId = this.findProcessDefinitionIdByDeploymentId(flowDeploymentId);
                tfCustomFlow.setFlowDefinitionId(processDefinitionId);
                tfCustomFlowService.updateById(tfCustomFlow);
            }
            flowResult = JSONObject.parseObject(tfCustomFlow.getFlowProcessVar(), FlowResult.class);
            if (tfCustomFlow.getType() == TFCustomTypeEnum.OFFICE_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.CLOUD_MARK_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.QUALITY_PROBLEM_FLOW) {
                map = this.getFlowFormPropertie(flowResult, map, 1);
            }
        } else if (Objects.nonNull(flowId)) {
            if (tfCustomFlow.getType() == TFCustomTypeEnum.OFFICE_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.CLOUD_MARK_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.QUALITY_PROBLEM_FLOW) {
                taskList = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).list();
                map = this.getFlowFormPropertie(map, taskList, flowResult, tfCustomFlow);
            }
        } else {
            if (tfCustomFlow.getType() == TFCustomTypeEnum.OFFICE_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.CLOUD_MARK_SOP_FLOW || tfCustomFlow.getType() == TFCustomTypeEnum.QUALITY_PROBLEM_FLOW) {
                taskList = taskService.createTaskQuery().taskId(String.valueOf(taskId)).list();
                map = this.getFlowFormPropertie(map, taskList, flowResult, tfCustomFlow);
            }
        }
        map.put(SystemConstant.FLOW_DEPLOYMENT_ID, tfCustomFlow.getFlowDeploymentId());
        map.put(SystemConstant.ID, tfCustomFlow.getId());
        return map;
    }

    /**
     * 获取用户待办
     *
     * @param iPage
     * @param userId
     * @return
     */
    @Override
    public IPage<WorkTaskResult> getUnDoneFlowTaskList(IPage<Map> iPage, Long userId) {
        Map<String, Object> map = new HashMap<>();
        IPage<WorkTaskResult> workTaskResultIPage = sysUserService.getUnDoneFlowTaskList(iPage, userId);
        if (Objects.nonNull(workTaskResultIPage) && !CollectionUtils.isEmpty(workTaskResultIPage.getRecords())) {
            for (WorkTaskResult w : workTaskResultIPage.getRecords()) {
                List<Task> taskList = taskService.createTaskQuery().taskId(String.valueOf(w.getTaskId())).list();
                if (!CollectionUtils.isEmpty(taskList)) {
                    for (Task t : taskList) {
                        FlowResult flowResult = JSONObject.parseObject(w.getFlowProcessVar(), FlowResult.class);
                        LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
                        FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(t.getTaskDefinitionKey())), FlowTaskResult.class);
                        map = this.getFlowFormPropertie(flowResult, map, currFlowTaskResult.getSetup());
                        w.setFlowProcessVar(null);
                        w.setSetupMap((FlowTaskResult) map.get(SystemConstant.FORM_PROPERTIES));
                    }
                }
            }
        }
        return workTaskResultIPage;
    }

    /**
     * 流程节点转他人审批
     *
     * @param userId
     * @param taskId
     * @return
     */
    @Override
    @Transactional
    public Boolean taskApproverExchange(Long userId, Long taskId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.equals(String.valueOf(sysUser.getId()), String.valueOf(userId))) {
            throw ExceptionResultEnum.ERROR.exception("不能转给自己！");
        }

        boolean lock = memoryLock.lock(SystemConstant.LOCK_FLOW_EXCHANGE_PREFIX + taskId, taskId, SystemConstant.LOCK_FLOW_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在转审中，请稍候再试！");
        }
        try {
            Task task = taskService.createTaskQuery().taskId(String.valueOf(taskId)).singleResult();
            Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_NO_DATA.exception());

            BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
            FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(task.getTaskDefinitionKey());
            boolean multiInstance = false;
            if (currFlow.getBehavior() instanceof MultiInstanceActivityBehavior) {//会签
                multiInstance = true;

                List<Task> taskList = taskService.createTaskQuery().processInstanceId(task.getProcessInstanceId()).list();
                List<String> taskApproveList = new ArrayList<>();
                for (Task t : taskList) {
                    if (Objects.equals(String.valueOf(t.getAssignee()), String.valueOf(userId))) {
                        throw ExceptionResultEnum.ERROR.exception("不能转给自己！");
                    }
                    taskApproveList.add(t.getAssignee());
                }
                if (taskApproveList.contains(userId)) {
                    throw ExceptionResultEnum.ERROR.exception("该审批人已有审批权限，无需流转");
                }
            } else {//或签
                Task taskUser = taskService.createTaskQuery().taskId(String.valueOf(taskId)).taskCandidateOrAssigned(String.valueOf(userId)).singleResult();
                if (Objects.nonNull(taskUser)) {
                    throw ExceptionResultEnum.ERROR.exception("该审批人已有审批权限，无需流转");
                }
            }

            if (!multiInstance) {
                taskService.deleteCandidateUser(String.valueOf(taskId), String.valueOf(sysUser.getId()));
                taskService.addCandidateUser(String.valueOf(taskId), String.valueOf(userId));
            } else {
                taskService.setAssignee(String.valueOf(taskId), String.valueOf(userId));
            }

            TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, Long.parseLong(task.getProcessInstanceId())));
            Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());

            FlowResult flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
            LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
            FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);
            TFFlowLog tfFlowLog = new TFFlowLog(sysUser.getOrgId(), sysUser.getId(), sysUser.getId(),
                    currFlowTaskResult.getSetup(), Long.parseLong(task.getProcessInstanceId()),
                    Long.parseLong(task.getId()), tfCustomFlowEntity.getId(), String.valueOf(userId));
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.EXCHANGE);
            SysUser exchangeUser = sysUserService.getById(userId);
            Optional.ofNullable(exchangeUser).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("转审人为空"));

            SysOrg sysOrg = commonCacheService.orgCache(sysUser.getOrgId());
            SysOrg exchangeOrg = commonCacheService.orgCache(exchangeUser.getOrgId());
            String orgName = Objects.nonNull(sysOrg) ? sysOrg.getName() : "无机构";
            String exchangeOrgName = Objects.nonNull(exchangeOrg) ? exchangeOrg.getName() : "无机构";
            tfFlowLog.setApproveRemark(sysUser.getRealName() + "(" + orgName + ")转" + exchangeUser.getRealName() + "(" + exchangeOrgName + ")审批");
            tfFlowLogService.save(tfFlowLog);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            memoryLock.unlock(SystemConstant.LOCK_FLOW_EXCHANGE_PREFIX + taskId);
        }
        return true;
    }

    /**
     * 流程终止
     *
     * @param flowId
     * @return
     */
    @Override
    @Transactional
    public Boolean flowEnd(Long flowId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        boolean lock = memoryLock.lock(SystemConstant.LOCK_FLOW_END_PREFIX + flowId, flowId, SystemConstant.LOCK_FLOW_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在终止中，请稍候再试！");
        }
        try {
            TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(new QueryWrapper<TFFlowApprove>().lambda().eq(TFFlowApprove::getFlowId, flowId));
            Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.FLOW_APPROVE_NO_DATA.exception());

            TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, flowId));
            Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());

            FlowResult flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
            LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
            List<Task> taskList = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).list();
            if (CollectionUtils.isEmpty(taskList)) {
                throw ExceptionResultEnum.FLOW_TASK_NO_DATA.exception();
            }
            FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(taskList.get(0).getTaskDefinitionKey())), FlowTaskResult.class);

            TFFlowLog tfFlowLog = new TFFlowLog(sysUser.getOrgId(), sysUser.getId(), sysUser.getId(),
                    currFlowTaskResult.getSetup(), flowId,
                    Long.parseLong(taskList.get(0).getId()), tfCustomFlowEntity.getId(), null);
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.END);
            tfFlowLog.setApproveRemark(sysUser.getRealName() + "终止流程");
            tfFlowLogService.save(tfFlowLog);

            tfFlowApprove.setStatus(FlowStatusEnum.END);
            tfFlowApprove.setSetup(FlowApproveSetupEnum.END.getSetup());
            tfFlowApproveService.updateById(tfFlowApprove);

            runtimeService.deleteProcessInstance(String.valueOf(flowId), "结束");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            memoryLock.unlock(SystemConstant.LOCK_FLOW_END_PREFIX + flowId);
        }
        return true;
    }

    /**
     * 获取流程详细信息接口
     *
     * @param flowId
     * @return
     */
    @Override
    public FlowViewResult flowView(Long flowId) {
        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, flowId));
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());

        //获取当前流程节点
        TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(new QueryWrapper<TFFlowApprove>().lambda().eq(TFFlowApprove::getFlowId, flowId));
        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.FLOW_APPROVE_NO_DATA.exception());

        FlowResult flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
        LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();

        FlowTaskResult currFlowTaskResult = null;
        List<FlowTaskResult> flowTaskHistoryList = null;
        List<Long> taskIdList = null;

        List<TFFlowViewLogResult> flowApproveHistoryList = null;
        Map<Integer, Integer> approveSetupMap = new HashMap<>();
        //获取流程审批历史
        List<TFFlowLogResult> tfFlowLogResultList = tfFlowLogService.findByObjectId(tfCustomFlowEntity.getId());
        if (!CollectionUtils.isEmpty(tfFlowLogResultList)) {
            flowApproveHistoryList = GsonUtil.fromJson(GsonUtil.toJson(tfFlowLogResultList), new TypeToken<List<TFFlowViewLogResult>>() {
            }.getType());
            for (TFFlowViewLogResult t : flowApproveHistoryList) {
                t.setMessageType(FlowMsgTypeEnum.STANDARD);
                approveSetupMap.put(t.getApproveSetup(), t.getApproveSetup());
            }
        }
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResult.getSetup() > 0) {
                //正在审批的表单
                if (Objects.nonNull(tfFlowApprove.getSetup()) && flowTaskResult.getSetup().intValue() == tfFlowApprove.getSetup().intValue()) {
                    currFlowTaskResult = flowTaskResult;
                    List<Task> taskList = taskService.createTaskQuery().processInstanceId(String.valueOf(flowId)).list();
                    if (!CollectionUtils.isEmpty(taskList)) {
                        taskIdList = taskList.stream().map(s -> Long.parseLong(s.getId())).collect(Collectors.toList());
                    }
                } else if (approveSetupMap.containsKey(flowTaskResult.getSetup())) {//已经审批过的表单
                    flowTaskHistoryList = CollectionUtils.isEmpty(flowTaskHistoryList) ? new ArrayList<>(approveSetupMap.size()) : flowTaskHistoryList;
                    flowTaskHistoryList.add(flowTaskResult);
                }
            }
        }
        return new FlowViewResult(flowId, tfFlowApprove.getStatus(), String.valueOf(tfFlowApprove.getId()), taskIdList, flowTaskHistoryList, currFlowTaskResult, flowApproveHistoryList);
    }

    /**
     * 获取表单属性
     *
     * @param map
     * @param taskList
     * @param flowResult
     * @param tfCustomFlow
     * @return
     */
    protected Map<String, Object> getFlowFormPropertie(Map<String, Object> map,
                                                       List<Task> taskList,
                                                       FlowResult flowResult,
                                                       TFCustomFlow tfCustomFlow) {
        if (CollectionUtils.isEmpty(taskList)) {
            throw ExceptionResultEnum.ERROR.exception("流程任务数据为空！");
        }
        Long flowId = Long.parseLong(taskList.get(0).getProcessInstanceId());
        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, flowId));
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());
        tfCustomFlow = tfCustomFlowService.getById(tfCustomFlowEntity.gettFCustomFlowId());
        Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.FLOW_DEPLOYMENT_NO_DATA.exception());

        flowResult = JSONObject.parseObject(tfCustomFlowEntity.getFlowProcessVar(), FlowResult.class);
        LinkedHashMap<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
        for (Task task : taskList) {
            FlowTaskResult currFlowTaskResult = GsonUtil.fromJson(GsonUtil.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);
            map = this.getFlowFormPropertie(flowResult, map, currFlowTaskResult.getSetup());
        }
        map.put(SystemConstant.FLOW_DEPLOYMENT_ID, tfCustomFlow.getFlowDeploymentId());
        map.put(SystemConstant.ID, tfCustomFlow.getId());
        return map;
    }

    /**
     * 获取表单属性
     *
     * @param flowResult
     * @param map
     * @param setup
     * @return
     */
    protected Map<String, Object> getFlowFormPropertie(FlowResult flowResult, Map<String, Object> map, int setup) {
        Map<String, FlowTaskResult> setupMap = flowResult.getSetupMap();
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == setup) {
                map.put(SystemConstant.FORM_PROPERTIES, flowTaskResultTemp);
                break;
            }
        }
        return map;
    }

    /**
     * 获取流程下一节点
     *
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param currSetup
     * @param setupMap
     * @param approvePass
     * @return
     */
    protected FlowTaskResult getNextFlowTaskResult(Integer nrOfCompletedInstances,
                                                   Integer nrOfInstances,
                                                   FlowTaskResult currFlowTaskResult,
                                                   FlowTaskResult nextFlowTaskResult,
                                                   int currSetup,
                                                   Map<String, FlowTaskResult> setupMap,
                                                   FlowApprovePassEnum approvePass) {
        if (approvePass == FlowApprovePassEnum.PASS || approvePass == FlowApprovePassEnum.START || approvePass == FlowApprovePassEnum.DRAFT) {
            if (Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() != nrOfInstances.intValue() || approvePass == FlowApprovePassEnum.DRAFT) {
                nextFlowTaskResult = currFlowTaskResult;
            } else {
                nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, currSetup, nextFlowTaskResult);
            }
        } else if (approvePass == FlowApprovePassEnum.REJECT || approvePass == FlowApprovePassEnum.CANCEL) {
            nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, currSetup, nextFlowTaskResult);
        }
        Optional.ofNullable(nextFlowTaskResult).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("该节点不能审批通过"));
        return nextFlowTaskResult;
    }

    /**
     * 获取下级审核节点
     *
     * @param setupMap
     * @param currSetup
     * @param nextFlowTaskResult
     * @return
     */
    protected FlowTaskResult getNextFlowTaskResult(Map<String, FlowTaskResult> setupMap,
                                                   int currSetup,
                                                   FlowTaskResult nextFlowTaskResult) {
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = GsonUtil.fromJson(GsonUtil.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == currSetup) {
                nextFlowTaskResult = flowTaskResultTemp;
                break;
            }
        }
        return nextFlowTaskResult;
    }

    /**
     * 审批通过
     *
     * @param currFlowTaskResult
     * @param setupMap
     * @param multiInstance
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param nextFlowTaskResult
     * @param map
     * @param approvePass
     * @param approveRemark
     */
    public void approvePass(FlowTaskResult currFlowTaskResult,
                            Map<String, FlowTaskResult> setupMap,
                            boolean multiInstance,
                            Integer nrOfCompletedInstances,
                            Integer nrOfInstances,
                            TFFlowApprove tfFlowApprove,
                            TFFlowLog tfFlowLog,
                            FlowTaskResult nextFlowTaskResult,
                            Map<String, Object> map,
                            FlowApprovePassEnum approvePass,
                            String approveRemark) {
        if (multiInstance) {
            map.computeIfAbsent(FlowApproveOperationEnum.REJECT.getId() + currFlowTaskResult.getSetup(), v -> 0);
        }
        if (currFlowTaskResult.getSetup().intValue() == setupMap.size() - 1) {
            if (multiInstance && Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() == nrOfInstances.intValue()) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else if (!multiInstance) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
            }
        } else {
            if (approvePass == FlowApprovePassEnum.DRAFT || approvePass == FlowApprovePassEnum.START) {
                tfFlowApprove.setStatus(FlowStatusEnum.valueOf(approvePass.name()));
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
            }
            tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
        }
        if (tfFlowApprove.getStatus() == FlowStatusEnum.START) {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.START);
        } else if (tfFlowApprove.getStatus() == FlowStatusEnum.DRAFT) {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.DRAFT);
        } else {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
        }
        if (Objects.nonNull(approveRemark)) {
            tfFlowLog.setApproveRemark(approveRemark);
        } else {
            tfFlowLog.setApproveRemark(tfFlowLog.getApproveOperation().getTitle());
        }
    }

    /**
     * 审批驳回
     *
     * @param multiInstance
     * @param map
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param approvePass
     * @param approveRemark
     */
    protected void approveReject(boolean multiInstance,
                                 Map<String, Object> map,
                                 FlowTaskResult currFlowTaskResult,
                                 FlowTaskResult nextFlowTaskResult,
                                 TFFlowApprove tfFlowApprove,
                                 TFFlowLog tfFlowLog,
                                 FlowApprovePassEnum approvePass,
                                 String approveRemark) {
        if (multiInstance) {
            map.computeIfAbsent(FlowApproveOperationEnum.REJECT.getId() + currFlowTaskResult.getSetup(), v -> 1);
        }
        tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
        tfFlowApprove.setStatus(FlowStatusEnum.convertToEnum(approvePass.name()));
        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.convertToEnum(approvePass.name()));
        if (Objects.nonNull(approveRemark)) {
            tfFlowLog.setApproveRemark(approveRemark);
        } else {
            tfFlowLog.setApproveRemark(tfFlowLog.getApproveOperation().getTitle());
        }
    }
}