package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.sop.business.bean.params.UserArchivesParam;
import com.qmth.sop.business.bean.result.UserArchivesResult;
import com.qmth.sop.business.bean.result.UserArchivesSubTotalResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBUserArchives;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.common.enums.RoleTypeEnum;

/**
 * <p>
 * 人员档案表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBUserArchivesService extends IService<TBUserArchives> {

    /**
     * 人员档案管理分页查询
     *
     * @param city                    城市
     * @param supplierId              供应桑id
     * @param archivesName            档案名称
     * @param roleType                认证橘色
     * @param authenticationStatus    认证状态
     * @param authenticationTimeStart 入档时间-起始
     * @param authenticationTimeEnd   入档时间-终止
     * @param remainValidDay          剩余有效天
     * @param pageNumber              分页页数
     * @param pageSize                分页容量
     * @return 结果
     */
    IPage<UserArchivesResult> findUserArchivesPage(String city, Long supplierId, String archivesName, RoleTypeEnum roleType,
                                                   Boolean authenticationStatus, Long authenticationTimeStart,
                                                   Long authenticationTimeEnd, Integer remainValidDay, Integer pageNumber, Integer pageSize);

    /**
     * 编辑用户档案
     *
     * @param userArchivesParam 人员档案参数
     * @param requestUser       请求的用户
     * @return 档案id
     */
    Long editUserArchives(UserArchivesParam userArchivesParam, SysUser requestUser);

    /**
     * 查询人员档案小计
     *
     * @return 档案小计情况
     */
    UserArchivesSubTotalResult findUserArchivesSubTotal();
}
