package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.SysCustomResult;
import com.qmth.sop.business.entity.SysCustom;
import com.qmth.sop.business.entity.SysCustomRole;
import com.qmth.sop.business.entity.SysLevelRole;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.mapper.SysCustomMapper;
import com.qmth.sop.business.service.SysCustomRoleService;
import com.qmth.sop.business.service.SysCustomService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Objects;

/**
 * 客户表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-02 11:55:19
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class SysCustomServiceImpl extends ServiceImpl<SysCustomMapper, SysCustom> implements SysCustomService {
    @Resource
    SysCustomRoleService sysCustomRoleService;

    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */
    @Override
    public IPage<SysCustomResult> query(Page<Map> iPage, ProductTypeEnum type, Long managerId, String name, Long levelId) {
        IPage<SysCustomResult> sysCustomResultIPage = this.baseMapper.query(iPage, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(managerId) ? managerId : null, name, Objects.nonNull(levelId) ? levelId : null);
        sysCustomResultIPage.getRecords().forEach(e -> {
            e.setRoleList(sysCustomRoleService.getList(e.getId()));
        });
        return  sysCustomResultIPage;
    }

    @Override
    @Transactional
    public void add(SysCustomResult sysCustom) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysCustom.setCreateId(sysUser.getId());
        sysCustom.setCreateTime(System.currentTimeMillis());
        this.saveOrUpdate(sysCustom);
        sysCustom.getRoleList().forEach(e->{
            e.setCustomId(sysCustom.getId());
            sysCustomRoleService.saveOrUpdate(e);
        });

    }

    @Override
    @Transactional
    public void update(SysCustomResult sysCustom) {
        sysCustomRoleService.remove(new QueryWrapper<SysCustomRole>().lambda().eq(SysCustomRole::getCustomId, sysCustom.getId()));
        sysCustom.getRoleList().forEach(e->{
            e.setCustomId(sysCustom.getId());
            sysCustomRoleService.saveOrUpdate(e);
        });
       
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysCustom.setUpdateId(sysUser.getId());
        sysCustom.setUpdateTime(System.currentTimeMillis());
        this.saveOrUpdate(sysCustom);
    }


    @Override
    @Transactional
    public void delete(Long id) {
        sysCustomRoleService.remove(new QueryWrapper<SysCustomRole>().lambda().eq(SysCustomRole::getCustomId, id));
        this.removeById(id);
    }


}