package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.SysDeviceResult;
import com.qmth.sop.business.entity.SysDevice;
import com.qmth.sop.business.entity.SysSupplier;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.mapper.SysDeviceMapper;
import com.qmth.sop.business.service.SysDeviceService;
import com.qmth.sop.common.enums.DeviceStatusEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 设备配置表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-02 14:21:11
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class SysDeviceServiceImpl extends ServiceImpl<SysDeviceMapper, SysDevice> implements SysDeviceService {


    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */
    @Override
    public <T> IPage<SysDeviceResult> query(Page<T> iPage, String query) {
        return this.baseMapper.query(iPage, query);
    }

    /**
     * 新增设备配置表
     *
     * @param sysDevice
     */
    @Override
    @Transactional
    public void add(SysDevice sysDevice) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysDevice.setCreateId(sysUser.getId());
        sysDevice.setCreateTime(System.currentTimeMillis());
        this.saveOrUpdate(sysDevice);
    }

    /**
     * 修改设备配置表
     *
     * @param sysDevice
     */
    @Override
    @Transactional
    public void update(SysDevice sysDevice) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        sysDevice.setUpdateId(sysUser.getId());
        sysDevice.setUpdateTime(System.currentTimeMillis());
        this.saveOrUpdate(sysDevice);
    }

    /**
     * 删除设备配置表
     *
     * @param id
     */
    @Override
    @Transactional
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    @Transactional
    public void disable(long id) {
        SysDevice sysDevice= this.getById(id);
        sysDevice.setEnable(false);
        this.update(sysDevice);
    }

}