package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.MenuResult;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.SysRole;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.SysUserRole;
import com.qmth.sop.business.mapper.SysUserRoleMapper;
import com.qmth.sop.business.service.SysUserRoleService;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.RoleTypeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户和角色关联表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
@Service
public class SysUserRoleServiceImpl extends ServiceImpl<SysUserRoleMapper, SysUserRole> implements SysUserRoleService {

    @Resource
    CommonCacheService commonCacheService;

    /**
     * 获取用户菜单
     *
     * @return
     */
    @Override
    public MenuResult listByUserId() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return commonCacheService.userMenuCache(sysUser.getId());
    }

    @Override
    public List<SysRole> listRoleByUserId(Long userId) {
        return this.baseMapper.listRoleByUserId(userId);
    }

    @Override
    public boolean userContainsRoles(Long userId, RoleTypeEnum... roleTypes) {
        boolean result = false;
        List<SysRole> sysRoleList = this.listRoleByUserId(userId);
        if (CollectionUtils.isNotEmpty(sysRoleList)) {
            List<RoleTypeEnum> targetRoleTypeList = Arrays.stream(roleTypes).collect(Collectors.toList());
            List<RoleTypeEnum> userRoleTypeList = sysRoleList.stream().map(SysRole::getType).collect(Collectors.toList());
            if (userRoleTypeList.containsAll(targetRoleTypeList)) {
                result = true;
            }
        }
        return result;
    }

    /**
     * 新增用户权限
     *
     * @param sysUser
     * @param roleIds
     */
    @Override
    @Transactional
    public void addUserRolePrivilege(SysUser sysUser, Long[] roleIds) {
        List<SysUserRole> sysUserRoleList = new ArrayList<>();
        for (int i = 0; i < roleIds.length; i++) {
            sysUserRoleList.add(new SysUserRole(sysUser.getId(), roleIds[i]));
        }
        this.saveBatch(sysUserRoleList);
    }

    /**
     * 根据角色id查询用户信息
     *
     * @param roleId
     * @return
     */
    @Override
    public List<SysUserRole> listByRoleId(Long roleId) {
        return this.baseMapper.listByRoleId(roleId);
    }
}
