package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.params.ServiceScopeParam;
import com.qmth.sop.business.bean.result.CrmServiceResult;
import com.qmth.sop.business.bean.result.CrmSubTotalResult;
import com.qmth.sop.business.bean.result.TBCrmResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBCrm;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.mapper.TBCrmMapper;
import com.qmth.sop.business.service.TBCrmService;
import com.qmth.sop.business.service.TBServiceService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FieldUniqueEnum;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.ServiceStatusEnum;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 派单信息表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBCrmServiceImpl extends ServiceImpl<TBCrmMapper, TBCrm> implements TBCrmService {
    @Resource
    private TBServiceService tbServiceService;

    @Override
    public IPage<CrmServiceResult> findServiceScopePage(Long serviceUnitId, String city, ProductTypeEnum productType, String customName, Boolean bindStatus, Integer pageNumber, Integer pageSize) {
        if (city != null && city.length() > 0) {
            city = SystemConstant.translateSpecificSign(city);
        }
        if (customName != null && customName.length() > 0) {
            customName = SystemConstant.translateSpecificSign(customName);
        }
        return this.baseMapper.findServiceScopePage(new Page<>(pageNumber, pageSize), serviceUnitId, city, productType, customName, bindStatus);
    }

    @Override
    public IPage<CrmServiceResult> findUnbindOrderPage(Long crmUserId, ProductTypeEnum productType, String customName, String crmNo, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        if (customName != null && customName.length() > 0) {
            customName = SystemConstant.translateSpecificSign(customName);
        }
        if (crmNo != null && crmNo.length() > 0) {
            crmNo = SystemConstant.translateSpecificSign(crmNo);
        }
        return this.baseMapper.findUnbindOrderPage(new Page<>(pageNumber, pageSize), crmUserId, productType, customName, crmNo, startTime, endTime);
    }

    @Override
    public List<CrmServiceResult> findCrmOrderByServiceIdList(List<Long> serviceIdList) {
        return this.baseMapper.findOrderListByServiceIds(serviceIdList);
    }

    @Transactional
    @Override
    public void bindCrmWithServiceBatch(ServiceScopeParam serviceScopeParam) {
        List<Long> crmIdList = serviceScopeParam.getCrmIdList();
        Long serviceUnitId = serviceScopeParam.getServiceUnitId();
        if (CollectionUtils.isEmpty(crmIdList)) {
            throw ExceptionResultEnum.ERROR.exception("未选择要划定的派单");
        }

        TBService tbService = tbServiceService.getById(serviceUnitId);
        if (Objects.isNull(tbService) || !tbService.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("服务单元不存在");
        } else {
            ServiceStatusEnum status = tbService.getStatus();
            if (ServiceStatusEnum.FINISH.equals(status) || ServiceStatusEnum.CANCEL.equals(status)) {
                throw ExceptionResultEnum.ERROR.exception("不能划定进已完结或已作废的服务单元");
            }
        }
        List<TBCrm> tbCrmList = this.listByIds(crmIdList);
        tbCrmList = tbCrmList.stream().peek(e -> e.setServiceId(serviceUnitId)).collect(Collectors.toList());
        this.updateBatchById(tbCrmList);
    }

    @Transactional
    @Override
    public void unbindCrmWithService(Long crmId) {
        TBCrm tbCrm = this.getById(crmId);
        if (Objects.isNull(tbCrm)) {
            throw ExceptionResultEnum.ERROR.exception("未找到派单信息");
        }
        Long serviceUnitId = tbCrm.getServiceId();
        TBService tbService = tbServiceService.getById(serviceUnitId);
        if (Objects.nonNull(tbService)) {
            ServiceStatusEnum status = tbService.getStatus();
            if (ServiceStatusEnum.FINISH.equals(status)) {
                throw ExceptionResultEnum.ERROR.exception("服务单元的当前状态更新为" + ServiceStatusEnum.FINISH.name() + "不允许移除");
            }
        }
        UpdateWrapper<TBCrm> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(TBCrm::getServiceId,null).eq(TBCrm::getId,tbCrm.getId());
        this.update(updateWrapper);
    }

    @Override
    public CrmSubTotalResult findCrmSubTotalData() {
        List<TBCrm> tbCrmList = this.list(new QueryWrapper<TBCrm>().lambda().eq(TBCrm::getEnable, true));
        int totalCrmCount = tbCrmList.size();
        int bindCrmCount = 0;
        int unbindCrmCount = 0;
        if (totalCrmCount > 0) {
            bindCrmCount = (int) tbCrmList.stream().filter(e -> e.getServiceId() != null).count();
            unbindCrmCount = (int) tbCrmList.stream().filter(e -> e.getServiceId() == null).count();
        }
        CrmSubTotalResult crmSubTotalResult = new CrmSubTotalResult();
        crmSubTotalResult.setTotalCrmCount(totalCrmCount);
        crmSubTotalResult.setBindCrmCount(bindCrmCount);
        crmSubTotalResult.setUnbindCrmCount(unbindCrmCount);
        return crmSubTotalResult;
    }

    @Override
    public IPage<TBCrmResult> query(IPage<Map> iPage, Long serviceId, Long leadId, Long crmUserId, ProductTypeEnum type, String custom, String crmNo, Long startTime, Long endTime) {
        return this.baseMapper.query(iPage ,serviceId,leadId,crmUserId,Objects.nonNull(type)?type.name():null,custom,crmNo,startTime,endTime);
    }


	/**
	 * 新增修改派单信息表
	 *
	 * @param tBCrm
	 */
	@Override
	@Transactional
	public Boolean saveTBCrm(TBCrm tBCrm) {
		try {
			SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
			if (Objects.isNull(tBCrm.getId())) {// 新增
				tBCrm.insertInfo(sysUser.getId());
			} else { // 修改
				tBCrm.updateInfo(sysUser.getId());
			}
			return saveOrUpdate(tBCrm);
		} catch (Exception e) {
			if (e instanceof DuplicateKeyException) {
				String errorColumn = e.getCause().toString();
				String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
				throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
			} else if (e instanceof ApiException) {
				ResultUtil.error((ApiException) e, e.getMessage());
			} else {
				ResultUtil.error(e.getMessage());
			}
		}
		return null;

	}

	/**
	 * 删除派单信息表
	 *
	 * @param id
	 */
	@Override
	@Transactional
	public Boolean delete(Long id) {
		return this.removeById(id);
	}

    @Override
    @Transactional
    public boolean batchZone(long serviceId, long[] crmIds) {
        List<Long> crmIdList = Arrays.stream(crmIds).boxed().collect(Collectors.toList());
        List<TBCrm> tbCrmList = this.listByIds(crmIdList);
        tbCrmList.forEach(e -> e.setServiceId(serviceId));
        return this.updateBatchById(tbCrmList);
    }

    @Override
    @Transactional
    public boolean batchDisable(long[] crmIds) {
        //批量修改
        List<Long> crmIdList = Arrays.stream(crmIds).boxed().collect(Collectors.toList());
        List<TBCrm> tbCrmList = this.listByIds(crmIdList);
        tbCrmList.forEach(e -> e.setEnable(false));
        return this.updateBatchById(tbCrmList);
    }

}
