package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.result.TBDelayWarnResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBDelayWarn;
import com.qmth.sop.business.mapper.TBDelayWarnMapper;
import com.qmth.sop.business.service.TBDelayWarnService;
import com.qmth.sop.common.enums.DelayWarnStatusEnum;
import com.qmth.sop.common.enums.DelayWarnTypeEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FieldUniqueEnum;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;
import java.util.Objects;

/**
 * 延期预警跟进表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-08 16:02:21
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class TBDelayWarnServiceImpl extends ServiceImpl<TBDelayWarnMapper, TBDelayWarn> implements TBDelayWarnService {


    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */
    @Override
    public IPage<TBDelayWarnResult> query(IPage<Map> iPage, DelayWarnTypeEnum type, DelayWarnStatusEnum status, Long serviceId, String custom, String fieldObj, String sopNo, Long startTime, Long endTime,Long id) {
        return this.baseMapper.query(iPage, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(status) ? status.name() : null, serviceId, custom, fieldObj, sopNo, startTime, endTime,id);
    }

    /**
     * 新增修改延期预警跟进表
     *
     * @param tBDelayWarn
     */
    @Override
    @Transactional
    public Boolean saveTBDelayWarn(TBDelayWarn tBDelayWarn) {
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (Objects.isNull(tBDelayWarn.getId())) {// 新增
                tBDelayWarn.insertInfo(sysUser.getId());
            } else { // 修改
                tBDelayWarn.updateInfo(sysUser.getId());
            }
            return saveOrUpdate(tBDelayWarn);
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return null;

    }

    /**
     * 删除延期预警跟进表
     *
     * @param id
     */
    @Override
    @Transactional
    public Boolean delete(Long id) {
        return this.removeById(id);
    }


}