package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.AreaDto;
import com.qmth.sop.business.entity.TBServiceRegionDetail;
import com.qmth.sop.business.mapper.TBServiceRegionDetailMapper;
import com.qmth.sop.business.service.TBServiceRegionDetailService;
import com.qmth.sop.common.contant.SystemConstant;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 服务区域规划明细表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBServiceRegionDetailServiceImpl extends ServiceImpl<TBServiceRegionDetailMapper, TBServiceRegionDetail> implements TBServiceRegionDetailService {

    @Transactional
    @Override
    public void bindServiceRegionAreas(Long serviceRegionId, List<AreaDto> areaDtoList) {
        // 删除之前绑定的区域
        this.remove(new QueryWrapper<TBServiceRegionDetail>().lambda().eq(TBServiceRegionDetail::getServiceRegionId, serviceRegionId));
        // 添加新绑定区域
        List<TBServiceRegionDetail> tbServiceRegionDetailList = areaDtoList.stream().flatMap(e -> {
            TBServiceRegionDetail detail = new TBServiceRegionDetail();
            detail.setId(SystemConstant.getDbUuid());
            detail.setServiceRegionId(serviceRegionId);
            detail.setCountry(e.getCountry());
            detail.setProvince(e.getProvince());
            detail.setCity(e.getCity());
            detail.setArea(e.getArea());
            detail.setAddress(e.getAddress());
            return Stream.of(detail);
        }).collect(Collectors.toList());
        this.saveBatch(tbServiceRegionDetailList);
    }
}
