package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.params.UserArchivesParam;
import com.qmth.sop.business.bean.result.UserArchivesResult;
import com.qmth.sop.business.bean.result.UserArchivesSubTotalResult;
import com.qmth.sop.business.entity.SysSupplier;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBUserArchives;
import com.qmth.sop.business.entity.TBUserArchivesSupplier;
import com.qmth.sop.business.mapper.TBUserArchivesMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.base.BaseEntity;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.EducationEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.GenderEnum;
import com.qmth.sop.common.enums.RoleTypeEnum;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 人员档案表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBUserArchivesServiceImpl extends ServiceImpl<TBUserArchivesMapper, TBUserArchives> implements TBUserArchivesService {
    @Resource
    private SysUserRoleService sysUserRoleService;
    @Resource
    private SysSupplierService sysSupplierService;
    @Resource
    private TBUserArchivesSupplierService tbUserArchivesSupplierService;
    @Resource
    private SysUserService sysUserService;

    @Override
    public IPage<UserArchivesResult> findUserArchivesPage(String city, Long supplierId, String archivesName, RoleTypeEnum roleType, Boolean authenticationStatus, Long authenticationTimeStart, Long authenticationTimeEnd, Integer remainValidDay, Integer pageNumber, Integer pageSize) {
        if (city != null && city.length() > 0) {
            city = SystemConstant.translateSpecificSign(city);
        }
        if (archivesName != null && archivesName.length() > 0) {
            archivesName = SystemConstant.translateSpecificSign(archivesName);
        }
        // 认证的角色枚举
        List<RoleTypeEnum> authRole = new ArrayList<>();
        authRole.add(RoleTypeEnum.EFFECT_ENGINEER);
        authRole.add(RoleTypeEnum.REGION_COORDINATOR);
        authRole.add(RoleTypeEnum.ASSISTANT_ENGINEER);

        Long remainValidTime = SystemConstant.processMiniute(Duration.ofDays(remainValidDay));
        IPage<UserArchivesResult> page = this.baseMapper.findUserArchivesPage(new Page<>(pageNumber, pageSize), city, supplierId, archivesName, roleType, authenticationStatus, authenticationTimeStart, authenticationTimeEnd, remainValidTime, System.currentTimeMillis());
        for (UserArchivesResult record : page.getRecords()) {
            List<RoleTypeEnum> roleTypeEnumList = new ArrayList<>();
            Long userId = record.getUserId();
            if (userId != null && userId > 0) {
                for (RoleTypeEnum roleTypeEnum : authRole) {
                    // TODO: 2023/8/14 可优化查询 
                    if (sysUserRoleService.userContainsRoles(userId, roleTypeEnum)) {
                        roleTypeEnumList.add(roleTypeEnum);
                    }
                }
            }
            record.setRoleTypeList(roleTypeEnumList);
        }
        return page;
    }

    @Transactional
    @Override
    public Long editUserArchives(UserArchivesParam userArchivesParam, SysUser requestUser) {
        Long id = userArchivesParam.getId();
        String code = userArchivesParam.getCode();
        // 校验档案编号唯一
        List<TBUserArchives> codeCheckList = this.list(new QueryWrapper<TBUserArchives>().lambda().eq(TBUserArchives::getCode, code));
        if (codeCheckList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("系统中存在多个相同的档案编号异常[" + code + "]");
        } else if (codeCheckList.size() == 1) {
            TBUserArchives codeCheck = codeCheckList.get(0);
            if (!Objects.equals(codeCheck.getId(), id)) {
                throw ExceptionResultEnum.ERROR.exception("系统中已经存在档案编号[" + code + "]");
            }
        }
        // 校验身份证号唯一
        String identify = userArchivesParam.getIdentity();
        List<TBUserArchives> identifyCheckList = this.list(new QueryWrapper<TBUserArchives>().lambda().eq(TBUserArchives::getIdentity, identify));
        if (identifyCheckList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("系统中存在多个相同的身份证号异常[" + identify + "]");
        } else if (identifyCheckList.size() == 1) {
            TBUserArchives identifyCheck = identifyCheckList.get(0);
            if (!Objects.equals(identifyCheck.getId(), id)) {
                throw ExceptionResultEnum.ERROR.exception("系统中已经存在身份证号[" + identify + "]");
            }
        }
        // 校验手机号唯一
        String mobileNumber = userArchivesParam.getMobileNumber();
        List<TBUserArchives> mobileNumberCheckList = this.list(new QueryWrapper<TBUserArchives>().lambda().eq(TBUserArchives::getMobileNumber, mobileNumber));
        if (mobileNumberCheckList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("系统中存在多个相同的手机号异常[" + mobileNumber + "]");
        } else if (mobileNumberCheckList.size() == 1) {
            TBUserArchives mobileNumberCheck = mobileNumberCheckList.get(0);
            if (!Objects.equals(mobileNumberCheck.getId(), id)) {
                throw ExceptionResultEnum.ERROR.exception("系统中已经存在手机号[" + mobileNumber + "]");
            }
        }

        String name = userArchivesParam.getName();
        String country = userArchivesParam.getCountry();
        String province = userArchivesParam.getProvince();
        String city = userArchivesParam.getCity();
        String area = userArchivesParam.getArea();
        String address = userArchivesParam.getAddress();
        GenderEnum gender = userArchivesParam.getGender();
        EducationEnum education = userArchivesParam.getEducation();
        String email = userArchivesParam.getEmail();
        String basePhotoPath = userArchivesParam.getBasePhotoPath();
        Long supplierId = userArchivesParam.getSupplierId();
        if (supplierId != null && supplierId > 0) {
            SysSupplier sysSupplier = sysSupplierService.getById(supplierId);
            if (Objects.isNull(sysSupplier)) {
                throw ExceptionResultEnum.ERROR.exception("供应商不存在");
            }
        }
        Long archivesTime = userArchivesParam.getArchivesTime();
        Long[] roleIds = userArchivesParam.getRoleIds();
        Long authenticationTime = userArchivesParam.getAuthenticationTime();
        Double authenticationScore = userArchivesParam.getAuthenticationScore();
        Long authenticationValidTime = userArchivesParam.getAuthenticationValidTime();
        String remark = userArchivesParam.getRemark();

        TBUserArchives tbUserArchives = new TBUserArchives();
        tbUserArchives.setCode(code);
        tbUserArchives.setName(name);
        tbUserArchives.setCountry(country);
        tbUserArchives.setProvince(province);
        tbUserArchives.setCity(city);
        tbUserArchives.setArea(area);
        tbUserArchives.setAddress(address);
        tbUserArchives.setIdentity(identify);
        tbUserArchives.setGender(gender);
        tbUserArchives.setEducation(education);
        tbUserArchives.setMobileNumber(mobileNumber);
        tbUserArchives.setEmail(email);
        tbUserArchives.setBasePhotoPath(basePhotoPath);
        // 档案表编辑
        Long userId = null;
        if (id == null || id == 0) {
            // 新增
            tbUserArchives.setCreateId(requestUser.getId());
            this.save(tbUserArchives);
            id = tbUserArchives.getId();
            List<SysUser> userList = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getMobileNumber, mobileNumber));
            if (CollectionUtils.isNotEmpty(userList)) {
                if (userList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception("用户表数据异常: 存在多个手机号相同的用户[" + mobileNumber + "]");
                }
                userId = userList.get(0).getId();
            }
        } else {
            // 编辑
            TBUserArchives old = this.getById(id);
            if (Objects.isNull(old)) {
                throw ExceptionResultEnum.ERROR.exception("用户不存在");
            }
            List<SysUser> userList = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getMobileNumber, old.getMobileNumber()));
            if (CollectionUtils.isNotEmpty(userList)) {
                if (userList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception("用户表数据异常: 存在多个手机号相同的用户[" + mobileNumber + "]");
                }
                userId = userList.get(0).getId();
            }
            tbUserArchives.setId(id);
            tbUserArchives.setUpdateId(requestUser.getId());
            this.updateById(tbUserArchives);
        }

        // 档案详细表编辑 （先根据档案id删除再新增）
        tbUserArchivesSupplierService.remove(new QueryWrapper<TBUserArchivesSupplier>().lambda().eq(TBUserArchivesSupplier::getUserArchivesId, id));
        TBUserArchivesSupplier tbUserArchivesSupplier = new TBUserArchivesSupplier();
        tbUserArchivesSupplier.setUserArchivesId(id);
        tbUserArchivesSupplier.setArchivesTime(archivesTime);
        tbUserArchivesSupplier.setAuthenticationTime(authenticationTime);
        tbUserArchivesSupplier.setAuthenticationScore(authenticationScore);
        tbUserArchivesSupplier.setAuthenticationValidTime(authenticationValidTime);
        tbUserArchivesSupplier.setSupplierId(supplierId);
        tbUserArchivesSupplier.setRemark(remark);
        tbUserArchivesSupplierService.save(tbUserArchivesSupplier);

        // 用户编辑 - 根据手机号关联更新
        SysUser sysUser = new SysUser();
        if (userId != null && userId > 0) {
            // 编辑
            sysUser = sysUserService.getById(userId);
        } else {
            sysUser.setLoginName(name);
            sysUser.setRealName(name);
            sysUser.setPasswordInfo(sysUser.getMobileNumber());
        }
        sysUser.setMobileNumber(mobileNumber);
        sysUser.setGender(gender);
        sysUser.setRoleIds(roleIds);
        sysUserService.saveUser(sysUser);
        return id;
    }

    @Override
    public UserArchivesSubTotalResult findUserArchivesSubTotal() {
        UserArchivesSubTotalResult result = new UserArchivesSubTotalResult();
        List<UserArchivesResult> userArchivesResultList = this.baseMapper.findSubTotal();
        if (CollectionUtils.isNotEmpty(userArchivesResultList)) {
            Long currentTime = System.currentTimeMillis();
            Integer totalCount = userArchivesResultList.size();
            List<UserArchivesResult> validList = userArchivesResultList.stream()
                    .filter(e -> e.getAuthenticationValidTime() != null && e.getAuthenticationValidTime() > currentTime)
                    .collect(Collectors.toList());
            Integer validCount = validList.size();
            Integer inValidCount = totalCount - validCount;

            result.setTotalCount(totalCount);
            result.setValidCount(validCount);
            result.setInvalidCount(inValidCount);

            List<Long> userIdList = validList.stream().map(UserArchivesResult::getUserId).distinct().collect(Collectors.toList());
            List<SysUser> effectEngineer = sysUserService.findUserListByRoleType(RoleTypeEnum.EFFECT_ENGINEER);
            if (CollectionUtils.isNotEmpty(effectEngineer)) {
                List<Long> effectEngineerIdList = effectEngineer.stream().map(BaseEntity::getId).distinct().collect(Collectors.toList());
                Integer effectEngineerCount = (int) userIdList.stream().filter(effectEngineerIdList::contains).count();
                result.setEffectEngineerCount(effectEngineerCount);
            }

            List<SysUser> regionCoordinator = sysUserService.findUserListByRoleType(RoleTypeEnum.REGION_COORDINATOR);
            if (CollectionUtils.isNotEmpty(regionCoordinator)) {
                List<Long> regionCoordinatorIdList = regionCoordinator.stream().map(BaseEntity::getId).distinct().collect(Collectors.toList());
                Integer effectEngineerCount = (int) userIdList.stream().filter(regionCoordinatorIdList::contains).count();
                result.setEffectEngineerCount(effectEngineerCount);
            }

            List<SysUser> assistantEngineer = sysUserService.findUserListByRoleType(RoleTypeEnum.ASSISTANT_ENGINEER);
            if (CollectionUtils.isNotEmpty(assistantEngineer)) {
                List<Long> assistantEngineerIdList = assistantEngineer.stream().map(BaseEntity::getId).distinct().collect(Collectors.toList());
                Integer assistantEngineerCount = (int) userIdList.stream().filter(assistantEngineerIdList::contains).count();
                result.setAssistantEngineerCount(assistantEngineerCount);
            }
        }
        return result;
    }
}