package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.result.TBViolationResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBViolation;
import com.qmth.sop.business.mapper.TBViolationMapper;
import com.qmth.sop.business.service.TBViolationService;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FieldUniqueEnum;
import com.qmth.sop.common.enums.ViolationStatusEnum;
import com.qmth.sop.common.enums.ViolationTypeEnum;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;
import java.util.Objects;

/**
 * sop违规登记表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-09 10:33:08
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class TBViolationServiceImpl extends ServiceImpl<TBViolationMapper, TBViolation> implements TBViolationService {


    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */


    @Override
    public IPage<TBViolationResult> query(IPage<Map> iPage, Long serviceId, ViolationTypeEnum type, ViolationStatusEnum status, Long userId, String custom, String sopNo, Long createId, Long startTime, Long endTime, Long id) {
        return this.baseMapper.query(iPage,serviceId,Objects.nonNull(type)?type.name():null,Objects.nonNull(status)?status.name():null,userId,custom,sopNo,createId,startTime,endTime,id);
    }

    /**
     * 新增修改sop违规登记表
     *
     * @param tBViolation
     */
    @Override
    @Transactional
    public Boolean saveTBViolation(TBViolation tBViolation) {
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (Objects.isNull(tBViolation.getId())) {// 新增
                tBViolation.insertInfo(sysUser.getId());
            } else { // 修改
                tBViolation.updateInfo(sysUser.getId());
            }
            return saveOrUpdate(tBViolation);
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return null;

    }

    /**
     * 删除sop违规登记表
     *
     * @param id
     */
    @Override
    @Transactional
    public Boolean delete(Long id) {
        return this.removeById(id);
    }

}