package com.qmth.sop.business.templete.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.sop.business.bean.dto.CustomImportDto;
import com.qmth.sop.business.bean.dto.SysMessageExportDto;
import com.qmth.sop.business.bean.dto.UserArchivesImportDto;
import com.qmth.sop.business.bean.result.SysCustomResult;
import com.qmth.sop.business.bean.result.SysMessageResult;
import com.qmth.sop.business.bean.result.TBDingCountQueryResult;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.service.*;
import com.qmth.sop.business.templete.service.TaskLogicService;
import com.qmth.sop.business.util.ImportExportUtil;
import com.qmth.sop.business.util.excel.BasicExcelListener;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.FlowStatusEnum;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.UploadFileEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.File;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    ImportExportUtil importExportUtil;

    @Resource
    BasicAttachmentService basicAttachmentService;


    @Resource
    SysCustomService sysCustomService;

    @Resource
    SysUserService sysUserService;


    @Resource
    SysLevelService sysLevelService;

    @Resource
    SysMessageService sysMessageService;
    @Resource
    TBDingService tbDingService;

    /**
     * 处理导入人员档案数据
     *
     * @param map 数据源
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public Map<String, Object> executeImportUserArchivesLogic(Map<String, Object> map) throws Exception {
        InputStream inputStream = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            inputStream = importExportUtil.getUploadFileInputStream(tbTask);
            LinkedMultiValueMap<String, UserArchivesImportDto> userArchivesImportDtoLinkedMultiValueMap = new LinkedMultiValueMap<>();
            StringJoiner errorData = new StringJoiner("");
            EasyExcel.read(inputStream, UserArchivesImportDto.class, new BasicExcelListener<UserArchivesImportDto>() {

                @Override
                public void handle(LinkedMultiValueMap<String, UserArchivesImportDto> dataList, StringJoiner errorDataSj) {
                    userArchivesImportDtoLinkedMultiValueMap.addAll(dataList);
                    errorData.add(errorDataSj.toString());
                }
            }).headRowNumber(2).sheet(0).doRead();

            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> userArchivesImportDtoLinkedMultiValueMap);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA_ERROR, v -> errorData.toString());

            //TODO 处理数据逻辑
        } finally {
            if (Objects.nonNull(inputStream)) {
                inputStream.close();
            }
        }
        return map;
    }


    /**
     * 处理客户表导入
     *
     * @param map 数据源
     * @return 结果
     * @throws Exception 异常
     */
    @Override
    @Transactional
    public Map<String, Object> executeImportCustomLogic(Map<String, Object> map) throws Exception {
        InputStream inputStream = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            inputStream = importExportUtil.getUploadFileInputStream(tbTask);
            LinkedMultiValueMap<String, CustomImportDto> customImportDtoLinkedMultiValueMap = new LinkedMultiValueMap<>();
            StringJoiner errorData = new StringJoiner("");
            EasyExcel.read(inputStream, CustomImportDto.class, new BasicExcelListener<CustomImportDto>() {

                @Override
                public void handle(LinkedMultiValueMap<String, CustomImportDto> dataList, StringJoiner errorDataSj) {
                    customImportDtoLinkedMultiValueMap.addAll(dataList);
                    errorData.add(errorDataSj.toString());
                }
            }).headRowNumber(2).sheet(0).doRead();


            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> customImportDtoLinkedMultiValueMap);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA_ERROR, v -> errorData.toString());
            int errorSize = Objects.nonNull(customImportDtoLinkedMultiValueMap.get(BasicExcelListener.ERROR)) ? Objects.requireNonNull(customImportDtoLinkedMultiValueMap.get(BasicExcelListener.ERROR)).size() : 0;
            if (errorSize == 0) {
                Objects.requireNonNull(customImportDtoLinkedMultiValueMap.get(BasicExcelListener.SUCCESS)).forEach(e -> {
                    SysCustom sysCustom = new SysCustom();
                    sysCustom.setAddress(e.getAddress());
                    sysCustom.setName(e.getName());
                    sysCustom.setArea(e.getArea());
                    sysCustom.setCity(e.getCity());
                    sysCustom.setCode(e.getCode());
                    ProductTypeEnum type=ProductTypeEnum.OFFICE.getTitle().equals(e.getType())?ProductTypeEnum.OFFICE:ProductTypeEnum.CLOUD_MARK;
                    sysCustom.setType(type);
                    sysCustom.setProvince(e.getProvince());
                    sysCustom.setPeoperDay(e.getPeoperDay());
                    SysUser sysUser = sysUserService.getOne(new QueryWrapper<SysUser>().lambda().eq(SysUser::getRealName, e.getManager()));
                    sysCustom.setManagerId(sysUser.getId());
                    SysLevel sysLevel = sysLevelService.getOne(new QueryWrapper<SysLevel>().lambda().eq(SysLevel::getLevel, e.getLevel()).eq(SysLevel::getType, type));
                    sysCustom.setLevelId(sysLevel.getId());
                    sysCustom.setSync(false);
                    sysCustom.setEnable(true);
                    SysUser ssysUser = (SysUser) ServletUtil.getRequestUser();
                    sysCustom.setCreateId(ssysUser.getId());
                    sysCustom.setCreateTime(System.currentTimeMillis());
                    sysCustomService.saveOrUpdate(sysCustom);

                });
            }


        } finally {
            if (Objects.nonNull(inputStream)) {
                inputStream.close();
            }
        }
        return map;
    }


    /**
     * 处理回执明细导出
     *
     * @param map 数据源
     * @return 结果
     * @throws Exception 异常
     */
    @Override
    public Map<String, Object> executeExportSysMessageLogic(Map<String, Object> map) throws Exception {
        File fileTemp = null;
        try {
            fileTemp = SystemConstant.getFileTempVar(SystemConstant.XLSX_PREFIX);

            List<SysMessageResult> lists = sysMessageService.query((Long) map.get("noticeId"), (String) map.get("city"), (Long) map.get("supplierId"), (Boolean) map.get("status"));
          //List<SysMessageResult> 转化成 List<SysMessageExportDto>

            List<SysMessageExportDto> sysMessageExportDtoList = lists.stream().map(e->{
                SysMessageExportDto dto=new SysMessageExportDto();
                dto.setCity(e.getCity());
                dto.setCode(e.getCode());
                dto.setCountry(e.getCountry());
                dto.setCustom(e.getCustom());
                dto.setMobileNumber(e.getMobileNumber());
                dto.setName(e.getName());
                dto.setProvince(e.getProvince());
                dto.setService(e.getService());
                dto.setStatus(e.getStatus()?"已阅":"未阅");
                dto.setSupplier(e.getSupplier());
                //时间戳转化为日期字符串
                dto.setTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(e.getReceiveTime()));
                dto.setType(e.getType().getTitle());
                return  dto;
            }).collect(Collectors.toList());

            EasyExcel.write(fileTemp, SysMessageExportDto.class).sheet("回执明细导出").doWrite(sysMessageExportDtoList);

            BasicAttachment basicAttachment = basicAttachmentService.saveAttachment(fileTemp, UploadFileEnum.FILE);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> sysMessageExportDtoList);
            map.computeIfAbsent(SystemConstant.EXCEL_ATTACHMENT, v -> basicAttachment);
        } finally {
            if (Objects.nonNull(fileTemp)) {
                fileTemp.delete();
            }
        }
        return map;
    }

    @Override
    public Map<String, Object> executeExportDingCountLogic(Map<String, Object> map) throws Exception {
        File fileTemp = null;
        try {
            fileTemp = SystemConstant.getFileTempVar(SystemConstant.XLSX_PREFIX);

            List<TBDingCountQueryResult> tbDingCountQueryResults = tbDingService.query((Long) map.get("serviceId"), (String) map.get("status"), (Long) map.get("createId"), (String) map.get("userName"), (Long) map.get("supplierId"), (String) map.get("String"), (String) map.get("sopNo"), (Long) map.get("days"));
            tbDingCountQueryResults.forEach(e->{
                e.setStatus(FlowStatusEnum.convertToEnum(e.getStatus()).getTitle());
            });
            EasyExcel.write(fileTemp, TBDingCountQueryResult.class).sheet("考勤打卡统计导出").doWrite(tbDingCountQueryResults);

            BasicAttachment basicAttachment = basicAttachmentService.saveAttachment(fileTemp, UploadFileEnum.FILE);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> tbDingCountQueryResults);
            map.computeIfAbsent(SystemConstant.EXCEL_ATTACHMENT, v -> basicAttachment);
        } finally {
            if (Objects.nonNull(fileTemp)) {
                fileTemp.delete();
            }
        }
        return map;
    }

    /**
     * 处理导出人员档案数据
     *
     * @param map 数据源
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public Map<String, Object> executeExportUserArchivesLogic(Map<String, Object> map) throws Exception {
        File fileTemp = null;
        try {
            fileTemp = SystemConstant.getFileTempVar(SystemConstant.XLSX_PREFIX);
            //TODO 数据读取逻辑start
            List<UserArchivesImportDto> archivesImportDtoList = new ArrayList<>();
            archivesImportDtoList.add(new UserArchivesImportDto("1", "2", "123", "aaa", "bbb"));
            archivesImportDtoList.add(new UserArchivesImportDto("4", "5", "789", "ccc", "ddd"));
            //TODO 数据读取逻辑end
            EasyExcel.write(fileTemp, UserArchivesImportDto.class).sheet("人员档案导出").doWrite(archivesImportDtoList);

            BasicAttachment basicAttachment = basicAttachmentService.saveAttachment(fileTemp, UploadFileEnum.FILE);
            map.computeIfAbsent(SystemConstant.EXCEL_DATA, v -> archivesImportDtoList);
            map.computeIfAbsent(SystemConstant.EXCEL_ATTACHMENT, v -> basicAttachment);
        } finally {
            if (Objects.nonNull(fileTemp)) {
                fileTemp.delete();
            }
        }
        return map;
    }


}
