package com.qmth.sop.server.api;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.segments.MergeSegments;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.sop.business.bean.result.EditResult;
import com.qmth.sop.business.bean.result.SysDeviceResult;
import com.qmth.sop.business.entity.SysDevice;
import com.qmth.sop.business.entity.SysLevel;
import com.qmth.sop.business.entity.TFCustomFlow;
import com.qmth.sop.business.service.SysDeviceService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.Objects;

/**
 * 设备配置表 控制器.
 * 
 * @author: shudonghui
 * @date: 2023-08-02 14:37:23
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Api(tags = "设备配置表 Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX+SystemConstant.PREFIX_URL_DEVICE)
public class SysDeviceController {


    @Resource
    SysDeviceService sysDeviceService;

   
    @ApiOperation(value = "设备配置表查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "设备配置表列表信息", response = SysDeviceResult.class)})
    public Result query(
                @ApiParam(value = "查询条件", required = false) @RequestParam(required = false) String query,
                @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<SysDeviceResult> resultIPage = sysDeviceService.query(new Page<>(pageNumber, pageSize),query);

        return ResultUtil.ok(resultIPage);
    }

   
    @ApiOperation(value = "新增设备配置表接口")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "新增设备配置表信息", response = Object.class)})
    public Result add(@Valid @ApiParam(value = "设备配置表信息", required = true) @RequestBody(required = true) SysDevice sysDevice, BindingResult bindingResult) throws InterruptedException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        sysDeviceService.add(sysDevice);
        return ResultUtil.ok();
    }


   
    @ApiOperation(value = "修改设备配置表接口")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "修改设备配置表信息", response = Object.class)})
    public Result update(@Valid @ApiParam(value = "设备配置表信息", required = true) @RequestBody(required = true) SysDevice sysDevice, BindingResult bindingResult) throws InterruptedException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        sysDeviceService.update(sysDevice);
        return ResultUtil.ok();
    }

   
    @ApiOperation(value = "删除设备配置表接口")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除设备配置表信息", response = Object.class)})
    public Result delete(@ApiParam(value = "设备配置表信息", required = true) @RequestParam(required = true) long id) {
        sysDeviceService.delete(id);
        return ResultUtil.ok();
    }

    /*
     * 设备配置表列表
     */
   
    @ApiOperation(value = "设备配置表列表接口")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "设备配置表列表", response = SysDevice.class)})
    public Result list(@ApiParam(value = "查询条件", required = false) @RequestParam(required = false) Boolean enable) {
        return ResultUtil.ok(sysDeviceService.list(new QueryWrapper<SysDevice>().lambda().eq(Objects.nonNull(enable),SysDevice::getEnable, enable)));
    }

    /**
     * 停用设备配置
     */
    @ApiOperation(value = "停用设备配置接口")
    @RequestMapping(value = "/disable", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "停用设备配置", response = Object.class)})
    public Result disable(@ApiParam(value = "设备配置表信息", required = true) @RequestParam(required = true) long id) {
        sysDeviceService.disable(id);
        return ResultUtil.ok();
    }
    /**
     * 获取单个
     * @param id
     * @return
     */

    @ApiOperation(value = "获取单个设备配置接口")
    @RequestMapping(value = "/get", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "获取单个设备配置", response = SysDevice.class)})
    public Result get(@ApiParam(value = "设备配置表id", required = true) @RequestParam(required = true) long id) {
        return ResultUtil.ok(sysDeviceService.getById(id));
    }







}