package com.qmth.sop.server.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.bean.result.FlowTablePropResult;
import com.qmth.sop.business.bean.result.FlowTdFormWidgetResult;
import com.qmth.sop.business.bean.result.FormWidgetResult;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.TDFormWidget;
import com.qmth.sop.business.entity.TDTableProp;
import com.qmth.sop.business.service.TDFormWidgetService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.WidgetCodeEnum;
import com.qmth.sop.common.enums.WidgetTypeEnum;
import com.qmth.sop.common.util.GsonUtil;
import com.qmth.sop.common.util.JacksonUtil;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "系统控件Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_WIDGET)
@Aac(auth = BOOL.FALSE)
public class TDFormWidgetController {

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    TDFormWidgetService tdFormWidgetService;

    @ApiOperation(value = "控件查询列表")
    @ApiResponses({@ApiResponse(code = 200, message = "控件查询列表信息", response = FormWidgetResult.class)})
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list() {
        return ResultUtil.ok(commonCacheService.formWidgetCache());
    }

    @ApiOperation(value = "流程控件查询")
    @ApiResponses({@ApiResponse(code = 200, message = "流程控件查询", response = FlowFormWidgetResult.class)})
    @RequestMapping(value = "/flow/select", method = RequestMethod.POST)
    public Result flowSelect(@ApiParam(value = "开始id", required = true) @RequestParam Long beginId,
                             @ApiParam(value = "结束id", required = true) @RequestParam Long endId) {
        List<TDFormWidget> tdFormWidgetList = tdFormWidgetService.list(new QueryWrapper<TDFormWidget>().lambda().ge(TDFormWidget::getId, beginId).le(TDFormWidget::getId, endId).eq(TDFormWidget::getType, WidgetTypeEnum.FORM));
        List<FlowFormWidgetResult> flowFormWidgetResultList = null;
        if (!CollectionUtils.isEmpty(tdFormWidgetList)) {
            flowFormWidgetResultList = GsonUtil.fromJson(JacksonUtil.parseJson(tdFormWidgetList), new TypeToken<List<FlowFormWidgetResult>>() {
            }.getType());
            Set<FlowFormWidgetResult> flowFormWidgetResultSet = flowFormWidgetResultList.stream().filter(s -> s.getCode() == WidgetCodeEnum.TABLE).collect(Collectors.toSet());
            if (!CollectionUtils.isEmpty(flowFormWidgetResultSet)) {
                for (FlowFormWidgetResult f : flowFormWidgetResultSet) {
                    List<TDTableProp> tdTablePropList = commonCacheService.tablePropCache(f.getId());
                    if (!CollectionUtils.isEmpty(tdTablePropList)) {
                        List<FlowTablePropResult> flowTablePropResultList = GsonUtil.fromJson(JacksonUtil.parseJson(tdTablePropList), new TypeToken<List<FlowTablePropResult>>() {
                        }.getType());
                        for (FlowTablePropResult tf : flowTablePropResultList) {
                            TDFormWidget tdFormWidget = commonCacheService.formWidgetCache(tf.getEditWidgetId());
                            FlowTdFormWidgetResult flowTdFormWidgetResult = GsonUtil.fromJson(GsonUtil.toJson(tdFormWidget), FlowTdFormWidgetResult.class);
                            tf.setTdFormWidget(flowTdFormWidgetResult);
                        }
                        f.setTablePropList(flowTablePropResultList);
                    }
                }
            }
        }
        return ResultUtil.ok(flowFormWidgetResultList);
    }

    @ApiOperation(value = "测试客户类型列表")
    @ApiResponses({@ApiResponse(code = 200, message = "客户类型列表信息", response = Object.class)})
    @RequestMapping(value = "/custom/list", method = RequestMethod.POST)
    public Result customList() {
        List<Map> customListMap = new ArrayList<>();
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("code", "jwc");
        map.put("name", "教务处");
        customListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "2");
        map.put("code", "cloudMark");
        map.put("name", "云阅卷");
        customListMap.add(map);
        return ResultUtil.ok(customListMap);
    }

    @ApiOperation(value = "测试派单客户经理")
    @ApiResponses({@ApiResponse(code = 200, message = "测试派单客户经理", response = Object.class)})
    @RequestMapping(value = "/crm/manager", method = RequestMethod.POST)
    public Result crmManager() {
        List<Map> crmManagerListMap = new ArrayList<>();
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("login_name", "sysadmin");
        map.put("real_name", "sysadmin");
        map.put("role", "ADMIN");
        map.put("roleName", "系统管理员");
        crmManagerListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "2");
        map.put("login_name", "admin1");
        map.put("real_name", "admin1");
        map.put("role", "SCHOOL_ADMIN");
        map.put("roleName", "学校管理员");
        crmManagerListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "3");
        map.put("login_name", "cs1");
        map.put("real_name", "cs1");
        map.put("role", "TEACHER");
        map.put("roleName", "测试打杂的1");
        crmManagerListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "4");
        map.put("login_name", "cs2");
        map.put("real_name", "cs2");
        map.put("role", "TEACHER");
        map.put("roleName", "测试打杂的2");
        crmManagerListMap.add(map);
        return ResultUtil.ok(crmManagerListMap);
    }

    @ApiOperation(value = "测试客户名称")
    @ApiResponses({@ApiResponse(code = 200, message = "测试客户名称", response = Object.class)})
    @RequestMapping(value = "/school/list", method = RequestMethod.POST)
    public Result schoolList() {
        List<Map> schoolListMap = new ArrayList<>();
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("code", "whdx");
        map.put("name", "武汉大学");
        schoolListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "2");
        map.put("code", "hzkjdx");
        map.put("name", "华中科技大学");
        schoolListMap.add(map);
        return ResultUtil.ok(schoolListMap);
    }

    @ApiOperation(value = "测试项目母单编号")
    @ApiResponses({@ApiResponse(code = 200, message = "测试项目母单编号", response = Object.class)})
    @RequestMapping(value = "/crm/parent_no/list", method = RequestMethod.POST)
    public Result crmParentNoList() {
        List<Map> crmParentNoListMap = new ArrayList<>();
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("code", "CRM_000001");
        map.put("name", "武汉大学数学学院2023上半年教务处印刷");
        crmParentNoListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "2");
        map.put("code", "CRM_000002");
        map.put("name", "武汉大学物理学院2023上半年教务处印刷");
        crmParentNoListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "3");
        map.put("code", "CRM_000003");
        map.put("name", "华中科技大学2023研究生云阅卷");
        crmParentNoListMap.add(map);
        return ResultUtil.ok(crmParentNoListMap);
    }

    @ApiOperation(value = "测试实施产品")
    @ApiResponses({@ApiResponse(code = 200, message = "测试实施产品", response = Object.class)})
    @RequestMapping(value = "/product/list", method = RequestMethod.POST)
    public Result productList() {
        List<Map> productListMap = new ArrayList<>();
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("code", "QM_WHDX_SX_000001");
        map.put("name", "武汉大学数学学院2023上半年教务处印刷");
        productListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "2");
        map.put("code", "QM_WHDX_WL_000002");
        map.put("name", "武汉大学物理学院2023上半年教务处印刷");
        productListMap.add(map);

        map = new LinkedHashMap();
        map.put("id", "3");
        map.put("code", "QM_HZKJDX_CLOUD_MARK_000001");
        map.put("name", "华中科技大学2023研究生云阅卷");
        productListMap.add(map);
        return ResultUtil.ok(productListMap);
    }

    @ApiOperation(value = "测试CRM信息")
    @ApiResponses({@ApiResponse(code = 200, message = "测试CRM信息", response = Object.class)})
    @RequestMapping(value = "/crm/info", method = RequestMethod.POST)
    public Result crmInfo() {
        Map map = new LinkedHashMap();
        map.put("id", "1");
        map.put("crm_child_no", "CRM_CHILD_000001");
        map.put("exam_start_time", 1688194545000L);
        map.put("exam_end_time", 1689663345000L);
        map.put("remark", "测试CRM特殊要求及备注");
        map.put("project_type", "测试CRM项目类型");
        return ResultUtil.ok(map);
    }
}
